
# This file was generated by 'versioneer.py' (0.29) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

import json

version_json = '''
{
 "date": "2024-03-13T15:19:05-0400",
 "dirty": false,
 "error": null,
 "full-revisionid": "0e67d198af1eec830c2baef7a0088f4583616c4d",
 "version": "0.0.8"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)
